/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef IKONDRIVER_H
#define IKONDRIVER_H

#include <Driver.h>
#include "controllers/common/acquisition/gate/TimeGateCommon.h"
#include "controllers/common/acquisition/mode/NoTofAcquisitionCommon.h"
#include "controllers/common/acquisition/mode/TofAcquisitionCommon.h"
#include "controllers/common/acquisition/mode/KineticAcquisitionCommon.h"
#include "controllers/common/acquisition/detector/DetectorCommon.h"
#include "controllers/common/sample_environment/temperature/TemperatureCommon.h"

namespace ikon {

/**
 * Global Implementation of the vme card called IKon made in Ill. It needs a board called Men A201.
 */
class IKonDriver: public acquisition::NoTofAcquisitionCommon,
		public acquisition::TimeGateCommon,
		 public acquisition::KineticAcquisitionCommon,
		public acquisition::DetectorCommon,
		public temperature::TemperatureCommon{

	friend class RealIKonDriver;
	friend class PerfectIKonDriver;
	friend class SimulatedIKonDriver;

public:

	//! Driver type value
	static const std::string TYPE;
	static const std::string ON_COMMAND ;
	static const std::string OFF_COMMAND;
	/*!
	 * \brief Constructor
	 * \param[in] name the name of the device driver
	 */
	IKonDriver(const std::string& name);

	/*!
	 * \brief Destructor
	 */
	virtual ~IKonDriver();

	/*!
	 * \brief Method called for executing a command
	 *
	 * \param[in] command the command to apply on the controller
	 */
	virtual void execute(const std::string& aCommand);

	/*!
	 * \brief Calculate the progression state.
	 */
	void calculateProgression();


	Property<int32> height;
	Property<int32> width;
	Property<int32> hflip;
	Property<int32> vflip;
	Property<int32> acqmode;

	Property<int32> hbind;
	Property<int32> vbind;
	Property<int32> rotate;
	Property<int32> cooling;
	Property<int32> readout;
	Property<int32> shutopentime;
	Property<int32> shutclosetime;
	Property<int32> shutmode;
	Property<int32> hsspeed;
	Property<int32> bclamp;

	Property<int32> gain;

	Property<int32> tempstatus;
	Property<bool > modeFVB;
	Property<int32> vsspeed;

private:

	/*!
	 * \brief Method called before changing the data size property value
	 * \param[in] value the property value
	 * \throws CannotSetValue the value isn't corrected, or property couldn't be changed
	 */
	virtual void refreshDataSizeProperty(int32 value) throw (CannotSetValue);
	virtual void refreshModeProperty(int32 value) throw (CannotSetValue);

	/*!
	 * \brief compute global gate status
	 * \param[in] value the driver status
	 */
	virtual void computeGateStatus(int32 value);

	/*!
	 * \brief compute global acquisition status
	 * \param[in] value the driver status
	 */
	virtual void computeAcquisitionStatus(int32 value);

	/*!
	 * \brief compute global detector status
	 * \param[in] value the driver status
	 */
	virtual void computeDetectorStatus(int32 value);

	uint16* m_DataShort;

	/**
	 * flag for synchronise read
	 */
	bool m_SynchroniseReadActivated;
	bool m_SynchroniseReadTerminated;

};

}
#endif //IKONDRIVER_H
