/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "CAENCdDriver.h"

#include "drivers/global/DriversCommands.h"
#include "CAENCdDef.h"
#include "CAENCdState.h"
#include "PerfectCAENCdDriver.h"
#include "RealCAENCdDriver.h"

namespace caen_cd {

const std::string CAENCdDriver::TYPE = "caen_cd";

const std::string CAENCdDriver::WRITE_PARAMETERS_COMMAND = "writeParam";
const std::string CAENCdDriver::SET_MODE_COMMAND = "setAcqMode";
const std::string CAENCdDriver::RESET_COMMAND = "reset";
const std::string CAENCdDriver::CLEAR_COMMAND = "clear";

/*
 * Constructor
 */
CAENCdDriver::CAENCdDriver(const std::string& name) {

	driver::Vme::init(name);

	registerStates(new RealCAENCdDriver(this), new PerfectCAENCdDriver(this), new PerfectCAENCdDriver(this));

	// Global properties
	nbChannels.init(this, SAVE, "nb_channels");
	nbBits.init(this, SAVE, "nb_bits");
    model.init(this, NOSAVE, "model");
    channels.init(this, NOSAVE, "channels");
    serialNumber.init(this, NOSAVE, "serial_number");
    modelName.init(this, NOSAVE, "model_name");
    rocFirmware.init(this, NOSAVE, "roc_firmware");
    amcFirmware.init(this, NOSAVE, "amc_firmware");

    // Board properties
    linkType.init(this, SAVE, "link_type");
    acqMode.init(this, SAVE, "acq_mode");
    dppAcqMode.init(this, SAVE, "dpp_acq_mode");
    dppAcqModeStr.init(this, NOSAVE, "dpp_acq_mode_str");
    saveMode.init(this, SAVE, "save_mode");
    saveModeStr.init(this, NOSAVE, "save_mode_str");
    recordLength.init(this, SAVE, "record_length");
    ioLevel.init(this, SAVE, "io_level");
    triggerMode.init(this, SAVE, "trigger_mode");
    channelMask.init(this, SAVE, "channel_mask");

	// Channels properties
    triggerHoldOff.init(this, SAVE, "trigger_holdoff");
    triggerThreshold.init(this, SAVE, "trigger_threshold");
    selfTrigger.init(this, SAVE, "self_trigger");
    chargeSensitivity.init(this, SAVE, "charge_sensitivity");
    shortGate.init(this, SAVE, "short_gate");
    longGate.init(this, SAVE, "long_gate");
    preGate.init(this, SAVE, "pre_gate");
    triggerValidationWindow.init(this, SAVE, "trigger_validation");
    baselineMean.init(this, SAVE, "base_mean");

	channelActive.init(this, SAVE, "channel_active");
	channelActiveStr.init(this, NOSAVE, "channel_active_str");
	dcOffset.init(this, SAVE, "dc_offset");
	pulsePolarity.init(this, SAVE, "pulse_polarity");
	pulsePolarityStr.init(this, NOSAVE, "pulse_polarity_str");
	preTrigger.init(this, SAVE, "pre_trigger");

	// Init the device command list
	initCommand(driver::INIT_COMMAND);
	initCommand(driver::READ_INFOS_COMMAND);
	initCommand(WRITE_PARAMETERS_COMMAND);
	initCommand(SET_MODE_COMMAND);
	initCommand(RESET_COMMAND);
	initCommand(CLEAR_COMMAND);

	// Init functions
	registerFunction(NONE_FUNCTION);

	addressType.update(driver::Vme::A32_ACCESS);
	deviceType = LEAF_DEVICE_TYPE_DEVICE_CONTAINER;

	registerRefresher(nbChannels, &CAENCdDriver::refreshNbChannelsProperty, this);
	registerRefresher(dppAcqMode, &CAENCdDriver::refreshDppAcqModeProperty, this);
	registerRefresher(saveMode, &CAENCdDriver::refreshSaveModeProperty, this);
	registerRefresher(channelActive, &CAENCdDriver::refreshChannelActiveProperty, this);
	registerRefresher(pulsePolarity, &CAENCdDriver::refreshPulsePolarityProperty, this);
}

/*
 * Destructor
 */
CAENCdDriver::~CAENCdDriver() {

}

/*
 * execute
 */
void CAENCdDriver::execute(const std::string& aCommand) {

	if ((aCommand != driver::STOP_COMMAND) && (aCommand != driver::STATUS_COMMAND)
			&& (aCommand != driver::READ_COMMAND)) {
		commandProgression = PROGRESSION_UNKNOWNSTATE_DEVICE_CONTAINER;
	}

	CAENCdState* currentState = dynamic_cast<CAENCdState*>(getCurrentState());
	// Check command
	if (aCommand == driver::INIT_COMMAND) {
		// Init command
		currentState->init();
	} else if (aCommand == WRITE_PARAMETERS_COMMAND) {
		// writeParam command
		currentState->writeParam();
		commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
	} else if (aCommand == SET_MODE_COMMAND) {
		// clear command
		currentState->setAcqMode();
		commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
	} else if (aCommand == CLEAR_COMMAND) {
		// clear command
		currentState->clear();
		commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
	} else if (aCommand == RESET_COMMAND) {
		// clear command
		currentState->reset();
		commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
	} else if (aCommand == driver::READ_INFOS_COMMAND) {
		// Info command
		currentState->readInfos();
		commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
	} else {
		//Error bad command
		//Todo , normaly it will be detect it device action driver
	}
}

void CAENCdDriver::refreshNbChannelsProperty(int32 aValue)  throw (CannotSetValue){

    triggerThreshold.resize(aValue);
    selfTrigger.resize(aValue);
    chargeSensitivity.resize(aValue);
    shortGate.resize(aValue);
    longGate.resize(aValue);
    preGate.resize(aValue);
    triggerValidationWindow.resize(aValue);
    baselineMean.resize(aValue);

	channelActive.resize(aValue);
	channelActiveStr.resize(aValue);
	dcOffset.resize(aValue);
	preTrigger.resize(aValue);
	pulsePolarity.resize(aValue);
	pulsePolarityStr.resize(aValue);
}

void CAENCdDriver::refreshDppAcqModeProperty(int32 aValue) throw (CannotSetValue) {

	switch (aValue) {
	case 0:
		dppAcqModeStr = "Oscilloscope";
		break;
	case 1:
		dppAcqModeStr = "Histogram";
		break;
	}
}

void CAENCdDriver::refreshSaveModeProperty(int32 aValue) throw (CannotSetValue) {

	switch (aValue) {
	case 0:
		saveModeStr = "Energy";
		break;
	case 1:
		saveModeStr = "Time";
		break;
	case 2:
		saveModeStr = "Energy&Time";
		break;
	}
}

void CAENCdDriver::refreshChannelActiveProperty(int32 index, int32 aValue) throw (CannotSetValue) {

	switch (aValue) {
	case 0:
		channelActiveStr.set(index, "Inactive");
		break;
	case 1:
		channelActiveStr.set(index, "Active");
		break;
	}
}

void CAENCdDriver::refreshPulsePolarityProperty(int32 index, int32 aValue) throw (CannotSetValue) {

	switch (aValue) {
	case 0:
		pulsePolarityStr.set(index, "Positive");
		break;
	case 1:
		pulsePolarityStr.set(index, "Negative");
		break;
	}
}

}
